Previous: [[Installation]] | Next: [[Message Queue Component]]

------

> Please ntoe: What follows is a walkthrough guide. You'll get a good understanding of the possibilities of this addon by reading through the short pages in order.

Usage is very easy. Simply add the `{{message-queue}}` component to your application template and you're able to start showing notifications to the user!

Let's dig in in more detail...

- [Templates](#templates)
- [Displaying a message](#displaying-a-message)
  - [Timed messages](#timed-messages)
  - [Untimed messages](#untimed-messages)
  - [Static messages](#static-messages)

## Templates

Add the message queue component to your application template:

```hbs
{{!-- app/templates/application.hbs --}}

{{message-queue}}
```

You can actually add the queue to any template(s) you like. For example,  a `post` route:

```hbs
{{!-- app/templates/post.hbs --}}

{{message-queue}}
```

If you place `{{message-queue}}` in route-specific templates the transitions will be seamless - messages will continue to show through transitions regardless of which template you put the component in.


## Displaying a message


### Timed Messages

Timed messages will display for an interval of time before automatically disappearing.

To display a message you must send it to the message queue using `flashMessage`. The `flashMessage` method is injected onto all routes, controllers, and views.

Thus, from any route, controller, or view, you can call:

```js
this.flashMessage(type, content);
```

`type` and `content` are required. Both should be strings.

Other options can be passed to `flashMessage`. If you're passing more than `type` and `content` you must pass all options as an object. Here are the available options:

```js
this.flashMessage({
  content: 'Here is a message', // String
  duration: 2000, // Number in milliseconds
  type: 'success', // String
});
```

`content` and `type` are always required. `duration` is an optional argument specifying how long the message should be shown for.

The default duration is set on the component. See [[the queue component|Message Queue Component]] for more information about changing default options.

If there are no messages in the queue, the queue will automatically display your message. If there are one or more messages already in the queue, the queue will display each method in turn - first in, first out.


### Untimed Messages

Untimed messages will display indefinitely until the user clicks 'close' on the message.

You can display an untimed message by passing a duration of `0`.

```js
this.flashMessage({
  content: 'Here is an untimed message', // String
  duration: 0, // Number in milliseconds
  type: 'warning',
});
```

Like timed messages, untimed messages display in the `{{message-queue}}` component.


### Static messages

Static messages can be added to your templates using a the `{{flash-message}}` component. A static message is one that renders directly in your template - **not** via the `{{message-queue}}` component or `flashMessage()` method. For example:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{flash-message type='success' content='Congratulations! It worked'}}
```

`type` and `content` are required options. Static messages have no durations. They remain visible until the user chooses to close them. Hoewever, you could use the message component in conjunction with an if statement:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{#if loginError}}
  {{flash-message type='error' content=loginError}}
{{/if}}
```

Static messages can also be used as block components, which accept Handlebars content:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{#flash-message type='error'}}
  {{#link-to 'index'}}Error! Click here to go home{{/link-to}}
{{/flash-message}}
```

------

Previous: [[Installation]] | Next: [[Message Queue Component]]
